from tqdm import tqdm
import numpy as np

class TrainerChecker:
    def __init__(self, torch_trainer, tf_trainer,
                  train_metrics, test_metrics,
                  length, batch_size):
        self.torch_trainer = torch_trainer
        self.tf_trainer = tf_trainer
        self.train_metrics = train_metrics
        self.test_metrics = test_metrics
        self.length = length
        self.batch_size = batch_size       
        pass
    def check_train_step(self, tf_dataloader, torch_dataloader):
        torch_iter = iter(torch_dataloader)
        tf_iter = iter(tf_dataloader)
        for i in tqdm(range(self.length), total=self.length):
          torch_data, tf_data = next(torch_iter), next(tf_iter)
          tf_metrics = self.tf_trainer.train_step(tf_data)#['loss'].numpy()
          torch_metrics = self.torch_trainer.train_step(torch_data)#['loss'].detach().numpy()
          for metric_name in self.train_metrics:
            if not np.isclose(torch_metrics[metric_name], tf_metrics[metric_name]):
              print("Error: Different train_step !!\n")
              return False
        print("Info: Check train_step passed !!\n")
        return True

    def chek_test_step(self, tf_dataloader, torch_dataloader):  
        tf_metrics_names = [ metric.name for metric in self.tf_trainer.metrics]
        tf_metrics = self.tf_trainer.evaluate(tf_dataloader, steps=self.length)
        tf_metrics = dict(zip(tf_metrics_names , tf_metrics))
        torch_metrics = self.torch_trainer.evaluate(torch_dataloader, steps=self.length)
        if self.test_metrics[0] == 'full': # type: ignore
          self.test_metrics = set(torch_metrics.keys()) & set(tf_metrics.keys())
        for metric_name in self.test_metrics:
            if not np.isclose(torch_metrics[metric_name], tf_metrics[metric_name], rtol=1e-3, atol=1e-3):
              print("Error: Different test_step !!\n")
              print(metric_name, torch_metrics[metric_name], tf_metrics[metric_name])
              return False
        print("Info: Check test_step passed !!\n")
        return True


import torch
import numpy as np

def tf_wrename(w_name):
    return w_name.split(':')[0].replace('kernel','weight').replace('/','.')

## Get_weights
def get_torch_weights(model):
    return {w_name: np.transpose(weight) for w_name, weight in model.state_dict().items()}

def get_tf_weights(model):
    return {tf_wrename(w.name): w.numpy() for w in model.trainable_variables }

## Set_weights
def set_torch_weights(model, weights):
  torch_state_dict = {w_name: torch.from_numpy(weight) for w_name, weight in weights.items()}
  model.load_state_dict(torch_state_dict, strict=False)

def set_tf_weights(model, weights):
  assigned_weights = []
  weight_list = [ w.numpy() for w in model.weights ]
  weight_name_list = [tf_wrename(w.name) for w in model.weights ]
  for w_name, weight in weights.items():
    if w_name in weight_name_list:
      index = weight_name_list.index(w_name)
      assigned_weights.append(w_name)
      if weight.shape == weight_list[index].shape:
        weight_list[index] = weight
      else:
        weight_list[index] = np.transpose(weight)
  model.set_weights(weight_list)
  return assigned_weights

## W_TF2Torch
def W_TF2Torch(tf_model, torch_model):
    tf_weights = get_tf_weights(tf_model)
    torch_weights = get_torch_weights(torch_model)
    assigned_weights = set_torch_weights(torch_model, tf_weights)
    print('Info: missing_tf_weights : ', [name for name in tf_weights.keys() if name not in assigned_weights])
    print('Info: missing_torch_weights : ', [name for name in torch_weights.keys() if name not in assigned_weights])

def W_Torch2TF(torch_model, tf_model):
    tf_weights = get_tf_weights(tf_model)
    torch_weights = get_torch_weights(torch_model)
    assigned_weights = set_tf_weights(tf_model, torch_weights)
    print('Info: missing_tf_weights : ', [name for name in tf_weights.keys() if name not in assigned_weights])
    print('Info: missing_torch_weights : ', [name for name in torch_weights.keys() if name not in assigned_weights])